==> opensearch-java-2.9.0-sources.jar: org/opensearch/client/opensearch/core/FieldCapsRequest.java <==
/*
 * SPDX-License-Identifier: Apache-2.0
 *
 * The OpenSearch Contributors require contributions made to
 * this file be licensed under the Apache-2.0 license or a
 * compatible open source license.
 */

/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/*
 * Modifications Copyright OpenSearch Contributors. See
 * GitHub history for details.
 */

package org.opensearch.client.opensearch.core;

import jakarta.json.stream.JsonGenerator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;
import javax.annotation.Nullable;
import org.opensearch.client.json.JsonpDeserializable;
import org.opensearch.client.json.JsonpDeserializer;
import org.opensearch.client.json.JsonpMapper;
import org.opensearch.client.json.JsonpSerializable;
import org.opensearch.client.json.ObjectBuilderDeserializer;
import org.opensearch.client.json.ObjectDeserializer;
import org.opensearch.client.opensearch._types.ErrorResponse;
import org.opensearch.client.opensearch._types.ExpandWildcard;
import org.opensearch.client.opensearch._types.RequestBase;
import org.opensearch.client.opensearch._types.mapping.RuntimeField;
import org.opensearch.client.opensearch._types.query_dsl.Query;
import org.opensearch.client.transport.Endpoint;
import org.opensearch.client.transport.endpoints.SimpleEndpoint;
import org.opensearch.client.util.ApiTypeHelper;
import org.opensearch.client.util.ObjectBuilder;
import org.opensearch.client.util.ObjectBuilderBase;

// typedef: _global.field_caps.Request

/**
 * Returns the information about the capabilities of fields among multiple
 * indices.
 *
 */
@JsonpDeserializable
public class FieldCapsRequest extends RequestBase implements JsonpSerializable {
    @Nullable
    private final Boolean allowNoIndices;

    private final List<ExpandWildcard> expandWildcards;

    private final List<String> fields;

    @Nullable
    private final Boolean ignoreUnavailable;

    @Nullable
    private final Boolean includeUnmapped;

    private final List<String> index;

    @Nullable
    private final Query indexFilter;

    private final Map<String, RuntimeField> runtimeMappings;

    // ---------------------------------------------------------------------------------------------

    private FieldCapsRequest(Builder builder) {

        this.allowNoIndices = builder.allowNoIndices;
        this.expandWildcards = ApiTypeHelper.unmodifiable(builder.expandWildcards);
        this.fields = ApiTypeHelper.unmodifiable(builder.fields);
        this.ignoreUnavailable = builder.ignoreUnavailable;
        this.includeUnmapped = builder.includeUnmapped;
        this.index = ApiTypeHelper.unmodifiable(builder.index);
        this.indexFilter = builder.indexFilter;
        this.runtimeMappings = ApiTypeHelper.unmodifiable(builder.runtimeMappings);

    }

    public static FieldCapsRequest of(Function<Builder, ObjectBuilder<FieldCapsRequest>> fn) {
        return fn.apply(new Builder()).build();
    }

    /**
     * Whether to ignore if a wildcard indices expression resolves into no concrete
     * indices. (This includes <code>_all</code> string or when no indices have been
     * specified)
     * <p>
     * API name: {@code allow_no_indices}
     */
    @Nullable
    public final Boolean allowNoIndices() {
        return this.allowNoIndices;
    }

    /**
     * Whether to expand wildcard expression to concrete indices that are open,
     * closed or both.
     * <p>
     * API name: {@code expand_wildcards}
     */
    public final List<ExpandWildcard> expandWildcards() {
        return this.expandWildcards;
    }

    /**
     * A comma-separated list of field names
     * <p>
     * API name: {@code fields}
     */
    public final List<String> fields() {
        return this.fields;
    }

    /**
     * Whether specified concrete indices should be ignored when unavailable
     * (missing or closed)
     * <p>
     * API name: {@code ignore_unavailable}
     */
    @Nullable
    public final Boolean ignoreUnavailable() {
        return this.ignoreUnavailable;
    }

    /**
     * Indicates whether unmapped fields should be included in the response.
     * <p>
     * API name: {@code include_unmapped}
     */
    @Nullable
    public final Boolean includeUnmapped() {
        return this.includeUnmapped;
    }

    /**
     * A comma-separated list of index names; use <code>_all</code> or empty string
     * to perform the operation on all indices
     * <p>
     * API name: {@code index}
     */
    public final List<String> index() {
        return this.index;
    }

    /**
     * API name: {@code index_filter}
     */
    @Nullable
    public final Query indexFilter() {
        return this.indexFilter;
    }

    /**
     * API name: {@code runtime_mappings}
     */
    public final Map<String, RuntimeField> runtimeMappings() {
        return this.runtimeMappings;
    }

    /**
     * Serialize this object to JSON.
     */
    public void serialize(JsonGenerator generator, JsonpMapper mapper) {
        generator.writeStartObject();
        serializeInternal(generator, mapper);
        generator.writeEnd();
    }

    protected void serializeInternal(JsonGenerator generator, JsonpMapper mapper) {

        if (this.indexFilter != null) {
            generator.writeKey("index_filter");
            this.indexFilter.serialize(generator, mapper);

        }
        if (ApiTypeHelper.isDefined(this.runtimeMappings)) {
            generator.writeKey("runtime_mappings");
            generator.writeStartObject();
            for (Map.Entry<String, RuntimeField> item0 : this.runtimeMappings.entrySet()) {
                generator.writeKey(item0.getKey());
                item0.getValue().serialize(generator, mapper);

            }
            generator.writeEnd();

        }

    }

    public Builder toBuilder() {
        return new Builder().allowNoIndices(allowNoIndices)
            .expandWildcards(expandWildcards)
            .fields(fields)
            .ignoreUnavailable(ignoreUnavailable)
            .includeUnmapped(includeUnmapped)
            .index(index)
            .indexFilter(indexFilter)
            .runtimeMappings(runtimeMappings);
    }

    // ---------------------------------------------------------------------------------------------

    /**
     * Builder for {@link FieldCapsRequest}.
     */

    public static class Builder extends ObjectBuilderBase implements ObjectBuilder<FieldCapsRequest> {
        @Nullable
        private Boolean allowNoIndices;

        @Nullable
        private List<ExpandWildcard> expandWildcards;

        @Nullable
        private List<String> fields;

        @Nullable
        private Boolean ignoreUnavailable;

        @Nullable
        private Boolean includeUnmapped;

        @Nullable
        private List<String> index;

        @Nullable
        private Query indexFilter;

        @Nullable
        private Map<String, RuntimeField> runtimeMappings;

        /**
         * Whether to ignore if a wildcard indices expression resolves into no concrete
         * indices. (This includes <code>_all</code> string or when no indices have been
         * specified)
         * <p>
         * API name: {@code allow_no_indices}
         */
        public final Builder allowNoIndices(@Nullable Boolean value) {
            this.allowNoIndices = value;
            return this;
        }

        /**
         * Whether to expand wildcard expression to concrete indices that are open,
         * closed or both.
         * <p>
         * API name: {@code expand_wildcards}
         * <p>
         * Adds all elements of <code>list</code> to <code>expandWildcards</code>.
         */
        public final Builder expandWildcards(List<ExpandWildcard> list) {
            this.expandWildcards = _listAddAll(this.expandWildcards, list);
            return this;
        }

        /**
         * Whether to expand wildcard expression to concrete indices that are open,
         * closed or both.
         * <p>
         * API name: {@code expand_wildcards}
         * <p>
         * Adds one or more values to <code>expandWildcards</code>.
         */
        public final Builder expandWildcards(ExpandWildcard value, ExpandWildcard... values) {
            this.expandWildcards = _listAdd(this.expandWildcards, value, values);
            return this;
        }

        /**
         * A comma-separated list of field names
         * <p>
         * API name: {@code fields}
         * <p>
         * Adds all elements of <code>list</code> to <code>fields</code>.
         */
        public final Builder fields(List<String> list) {
            this.fields = _listAddAll(this.fields, list);
            return this;
        }

        /**
         * A comma-separated list of field names
         * <p>
         * API name: {@code fields}
         * <p>
         * Adds one or more values to <code>fields</code>.
         */
        public final Builder fields(String value, String... values) {
            this.fields = _listAdd(this.fields, value, values);
            return this;
        }

        /**
         * Whether specified concrete indices should be ignored when unavailable
         * (missing or closed)
         * <p>
         * API name: {@code ignore_unavailable}
         */
        public final Builder ignoreUnavailable(@Nullable Boolean value) {
            this.ignoreUnavailable = value;
            return this;
        }

        /**
         * Indicates whether unmapped fields should be included in the response.
         * <p>
         * API name: {@code include_unmapped}
         */
        public final Builder includeUnmapped(@Nullable Boolean value) {
            this.includeUnmapped = value;
            return this;
        }

        /**
         * A comma-separated list of index names; use <code>_all</code> or empty string
         * to perform the operation on all indices
         * <p>
         * API name: {@code index}
         * <p>
         * Adds all elements of <code>list</code> to <code>index</code>.
         */
        public final Builder index(List<String> list) {
            this.index = _listAddAll(this.index, list);
            return this;
        }

        /**
         * A comma-separated list of index names; use <code>_all</code> or empty string
         * to perform the operation on all indices
         * <p>
         * API name: {@code index}
         * <p>
         * Adds one or more values to <code>index</code>.
         */
        public final Builder index(String value, String... values) {
            this.index = _listAdd(this.index, value, values);
            return this;
        }

        /**
         * API name: {@code index_filter}
         */
        public final Builder indexFilter(@Nullable Query value) {
            this.indexFilter = value;
            return this;
        }

        /**
         * API name: {@code index_filter}
         */
        public final Builder indexFilter(Function<Query.Builder, ObjectBuilder<Query>> fn) {
            return this.indexFilter(fn.apply(new Query.Builder()).build());
        }

        /**
         * API name: {@code runtime_mappings}
         * <p>
         * Adds all entries of <code>map</code> to <code>runtimeMappings</code>.
         */
        public final Builder runtimeMappings(Map<String, RuntimeField> map) {
            this.runtimeMappings = _mapPutAll(this.runtimeMappings, map);
            return this;
        }

        /**
         * API name: {@code runtime_mappings}
         * <p>
         * Adds an entry to <code>runtimeMappings</code>.
         */
        public final Builder runtimeMappings(String key, RuntimeField value) {
            this.runtimeMappings = _mapPut(this.runtimeMappings, key, value);
            return this;
        }

        /**
         * API name: {@code runtime_mappings}
         * <p>
         * Adds an entry to <code>runtimeMappings</code> using a builder lambda.
         */
        public final Builder runtimeMappings(String key, Function<RuntimeField.Builder, ObjectBuilder<RuntimeField>> fn) {
            return runtimeMappings(key, fn.apply(new RuntimeField.Builder()).build());
        }

        /**
         * Builds a {@link FieldCapsRequest}.
         *
         * @throws NullPointerException
         *             if some of the required fields are null.
         */
        public FieldCapsRequest build() {
            _checkSingleUse();

            return new FieldCapsRequest(this);
        }
    }

    // ---------------------------------------------------------------------------------------------

    /**
     * Json deserializer for {@link FieldCapsRequest}
     */
    public static final JsonpDeserializer<FieldCapsRequest> _DESERIALIZER = ObjectBuilderDeserializer.lazy(
        Builder::new,
        FieldCapsRequest::setupFieldCapsRequestDeserializer
    );

    protected static void setupFieldCapsRequestDeserializer(ObjectDeserializer<FieldCapsRequest.Builder> op) {

        op.add(Builder::indexFilter, Query._DESERIALIZER, "index_filter");
        op.add(Builder::runtimeMappings, JsonpDeserializer.stringMapDeserializer(RuntimeField._DESERIALIZER), "runtime_mappings");

    }

    // ---------------------------------------------------------------------------------------------

    /**
     * Endpoint "{@code field_caps}".
     */
    public static final Endpoint<FieldCapsRequest, FieldCapsResponse, ErrorResponse> _ENDPOINT = new SimpleEndpoint<>(

        // Request method
        request -> {
            return "POST";

        },

        // Request path
        request -> {
            final int _index = 1 << 0;

            int propsSet = 0;

            if (ApiTypeHelper.isDefined(request.index())) propsSet |= _index;

            if (propsSet == 0) {
                StringBuilder buf = new StringBuilder();
                buf.append("/_field_caps");
                return buf.toString();
            }
            if (propsSet == (_index)) {
                StringBuilder buf = new StringBuilder();
                buf.append("/");
                SimpleEndpoint.pathEncode(request.index.stream().map(v -> v).collect(Collectors.joining(",")), buf);
                buf.append("/_field_caps");
                return buf.toString();
            }
            throw SimpleEndpoint.noPathTemplateFound("path");

        },

        // Request parameters
        request -> {
            Map<String, String> params = new HashMap<>();
            if (ApiTypeHelper.isDefined(request.expandWildcards)) {
                params.put("expand_wildcards", request.expandWildcards.stream().map(v -> v.jsonValue()).collect(Collectors.joining(",")));
            }
            if (request.ignoreUnavailable != null) {
                params.put("ignore_unavailable", String.valueOf(request.ignoreUnavailable));
            }
            if (request.allowNoIndices != null) {
                params.put("allow_no_indices", String.valueOf(request.allowNoIndices));
            }
            if (ApiTypeHelper.isDefined(request.fields)) {
                params.put("fields", request.fields.stream().map(v -> v).collect(Collectors.joining(",")));
            }
            if (request.includeUnmapped != null) {
                params.put("include_unmapped", String.valueOf(request.includeUnmapped));
            }
            return params;

        },
        SimpleEndpoint.emptyMap(),
        true,
        FieldCapsResponse._DESERIALIZER
    );
}

==> opensearch-java-2.22.0-sources.jar: org/opensearch/client/opensearch/core/FieldCapsRequest.java <==
/*
 * SPDX-License-Identifier: Apache-2.0
 *
 * The OpenSearch Contributors require contributions made to
 * this file be licensed under the Apache-2.0 license or a
 * compatible open source license.
 */

/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/*
 * Modifications Copyright OpenSearch Contributors. See
 * GitHub history for details.
 */

package org.opensearch.client.opensearch.core;

import jakarta.json.stream.JsonGenerator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;
import javax.annotation.Nullable;
import org.opensearch.client.json.JsonpDeserializable;
import org.opensearch.client.json.JsonpDeserializer;
import org.opensearch.client.json.JsonpMapper;
import org.opensearch.client.json.ObjectBuilderDeserializer;
import org.opensearch.client.json.ObjectDeserializer;
import org.opensearch.client.json.PlainJsonSerializable;
import org.opensearch.client.opensearch._types.ErrorResponse;
import org.opensearch.client.opensearch._types.ExpandWildcard;
import org.opensearch.client.opensearch._types.RequestBase;
import org.opensearch.client.opensearch._types.mapping.RuntimeField;
import org.opensearch.client.opensearch._types.query_dsl.Query;
import org.opensearch.client.transport.Endpoint;
import org.opensearch.client.transport.endpoints.SimpleEndpoint;
import org.opensearch.client.util.ApiTypeHelper;
import org.opensearch.client.util.ObjectBuilder;
import org.opensearch.client.util.ObjectBuilderBase;

// typedef: _global.field_caps.Request

/**
 * Returns the information about the capabilities of fields among multiple
 * indices.
 *
 */
@JsonpDeserializable
public class FieldCapsRequest extends RequestBase implements PlainJsonSerializable {
    @Nullable
    private final Boolean allowNoIndices;

    private final List<ExpandWildcard> expandWildcards;

    private final List<String> fields;

    @Nullable
    private final Boolean ignoreUnavailable;

    @Nullable
    private final Boolean includeUnmapped;

    private final List<String> index;

    @Nullable
    private final Query indexFilter;

    private final Map<String, RuntimeField> runtimeMappings;

    // ---------------------------------------------------------------------------------------------

    private FieldCapsRequest(Builder builder) {

        this.allowNoIndices = builder.allowNoIndices;
        this.expandWildcards = ApiTypeHelper.unmodifiable(builder.expandWildcards);
        this.fields = ApiTypeHelper.unmodifiable(builder.fields);
        this.ignoreUnavailable = builder.ignoreUnavailable;
        this.includeUnmapped = builder.includeUnmapped;
        this.index = ApiTypeHelper.unmodifiable(builder.index);
        this.indexFilter = builder.indexFilter;
        this.runtimeMappings = ApiTypeHelper.unmodifiable(builder.runtimeMappings);

    }

    public static FieldCapsRequest of(Function<Builder, ObjectBuilder<FieldCapsRequest>> fn) {
        return fn.apply(new Builder()).build();
    }

    /**
     * Whether to ignore if a wildcard indices expression resolves into no concrete
     * indices. (This includes <code>_all</code> string or when no indices have been
     * specified)
     * <p>
     * API name: {@code allow_no_indices}
     */
    @Nullable
    public final Boolean allowNoIndices() {
        return this.allowNoIndices;
    }

    /**
     * Whether to expand wildcard expression to concrete indices that are open,
     * closed or both.
     * <p>
     * API name: {@code expand_wildcards}
     */
    public final List<ExpandWildcard> expandWildcards() {
        return this.expandWildcards;
    }

    /**
     * A comma-separated list of field names
     * <p>
     * API name: {@code fields}
     */
    public final List<String> fields() {
        return this.fields;
    }

    /**
     * Whether specified concrete indices should be ignored when unavailable
     * (missing or closed)
     * <p>
     * API name: {@code ignore_unavailable}
     */
    @Nullable
    public final Boolean ignoreUnavailable() {
        return this.ignoreUnavailable;
    }

    /**
     * Indicates whether unmapped fields should be included in the response.
     * <p>
     * API name: {@code include_unmapped}
     */
    @Nullable
    public final Boolean includeUnmapped() {
        return this.includeUnmapped;
    }

    /**
     * A comma-separated list of index names; use <code>_all</code> or empty string
     * to perform the operation on all indices
     * <p>
     * API name: {@code index}
     */
    public final List<String> index() {
        return this.index;
    }

    /**
     * API name: {@code index_filter}
     */
    @Nullable
    public final Query indexFilter() {
        return this.indexFilter;
    }

    /**
     * API name: {@code runtime_mappings}
     */
    public final Map<String, RuntimeField> runtimeMappings() {
        return this.runtimeMappings;
    }

    /**
     * Serialize this object to JSON.
     */
    public void serialize(JsonGenerator generator, JsonpMapper mapper) {
        generator.writeStartObject();
        serializeInternal(generator, mapper);
        generator.writeEnd();
    }

    protected void serializeInternal(JsonGenerator generator, JsonpMapper mapper) {

        if (this.indexFilter != null) {
            generator.writeKey("index_filter");
            this.indexFilter.serialize(generator, mapper);

        }
        if (ApiTypeHelper.isDefined(this.runtimeMappings)) {
            generator.writeKey("runtime_mappings");
            generator.writeStartObject();
            for (Map.Entry<String, RuntimeField> item0 : this.runtimeMappings.entrySet()) {
                generator.writeKey(item0.getKey());
                item0.getValue().serialize(generator, mapper);

            }
            generator.writeEnd();

        }

    }

    public Builder toBuilder() {
        return new Builder().allowNoIndices(allowNoIndices)
            .expandWildcards(expandWildcards)
            .fields(fields)
            .ignoreUnavailable(ignoreUnavailable)
            .includeUnmapped(includeUnmapped)
            .index(index)
            .indexFilter(indexFilter)
            .runtimeMappings(runtimeMappings);
    }

    // ---------------------------------------------------------------------------------------------

    /**
     * Builder for {@link FieldCapsRequest}.
     */

    public static class Builder extends ObjectBuilderBase implements ObjectBuilder<FieldCapsRequest> {
        @Nullable
        private Boolean allowNoIndices;

        @Nullable
        private List<ExpandWildcard> expandWildcards;

        @Nullable
        private List<String> fields;

        @Nullable
        private Boolean ignoreUnavailable;

        @Nullable
        private Boolean includeUnmapped;

        @Nullable
        private List<String> index;

        @Nullable
        private Query indexFilter;

        @Nullable
        private Map<String, RuntimeField> runtimeMappings;

        /**
         * Whether to ignore if a wildcard indices expression resolves into no concrete
         * indices. (This includes <code>_all</code> string or when no indices have been
         * specified)
         * <p>
         * API name: {@code allow_no_indices}
         */
        public final Builder allowNoIndices(@Nullable Boolean value) {
            this.allowNoIndices = value;
            return this;
        }

        /**
         * Whether to expand wildcard expression to concrete indices that are open,
         * closed or both.
         * <p>
         * API name: {@code expand_wildcards}
         * <p>
         * Adds all elements of <code>list</code> to <code>expandWildcards</code>.
         */
        public final Builder expandWildcards(List<ExpandWildcard> list) {
            this.expandWildcards = _listAddAll(this.expandWildcards, list);
            return this;
        }

        /**
         * Whether to expand wildcard expression to concrete indices that are open,
         * closed or both.
         * <p>
         * API name: {@code expand_wildcards}
         * <p>
         * Adds one or more values to <code>expandWildcards</code>.
         */
        public final Builder expandWildcards(ExpandWildcard value, ExpandWildcard... values) {
            this.expandWildcards = _listAdd(this.expandWildcards, value, values);
            return this;
        }

        /**
         * A comma-separated list of field names
         * <p>
         * API name: {@code fields}
         * <p>
         * Adds all elements of <code>list</code> to <code>fields</code>.
         */
        public final Builder fields(List<String> list) {
            this.fields = _listAddAll(this.fields, list);
            return this;
        }

        /**
         * A comma-separated list of field names
         * <p>
         * API name: {@code fields}
         * <p>
         * Adds one or more values to <code>fields</code>.
         */
        public final Builder fields(String value, String... values) {
            this.fields = _listAdd(this.fields, value, values);
            return this;
        }

        /**
         * Whether specified concrete indices should be ignored when unavailable
         * (missing or closed)
         * <p>
         * API name: {@code ignore_unavailable}
         */
        public final Builder ignoreUnavailable(@Nullable Boolean value) {
            this.ignoreUnavailable = value;
            return this;
        }

        /**
         * Indicates whether unmapped fields should be included in the response.
         * <p>
         * API name: {@code include_unmapped}
         */
        public final Builder includeUnmapped(@Nullable Boolean value) {
            this.includeUnmapped = value;
            return this;
        }

        /**
         * A comma-separated list of index names; use <code>_all</code> or empty string
         * to perform the operation on all indices
         * <p>
         * API name: {@code index}
         * <p>
         * Adds all elements of <code>list</code> to <code>index</code>.
         */
        public final Builder index(List<String> list) {
            this.index = _listAddAll(this.index, list);
            return this;
        }

        /**
         * A comma-separated list of index names; use <code>_all</code> or empty string
         * to perform the operation on all indices
         * <p>
         * API name: {@code index}
         * <p>
         * Adds one or more values to <code>index</code>.
         */
        public final Builder index(String value, String... values) {
            this.index = _listAdd(this.index, value, values);
            return this;
        }

        /**
         * API name: {@code index_filter}
         */
        public final Builder indexFilter(@Nullable Query value) {
            this.indexFilter = value;
            return this;
        }

        /**
         * API name: {@code index_filter}
         */
        public final Builder indexFilter(Function<Query.Builder, ObjectBuilder<Query>> fn) {
            return this.indexFilter(fn.apply(new Query.Builder()).build());
        }

        /**
         * API name: {@code runtime_mappings}
         * <p>
         * Adds all entries of <code>map</code> to <code>runtimeMappings</code>.
         */
        public final Builder runtimeMappings(Map<String, RuntimeField> map) {
            this.runtimeMappings = _mapPutAll(this.runtimeMappings, map);
            return this;
        }

        /**
         * API name: {@code runtime_mappings}
         * <p>
         * Adds an entry to <code>runtimeMappings</code>.
         */
        public final Builder runtimeMappings(String key, RuntimeField value) {
            this.runtimeMappings = _mapPut(this.runtimeMappings, key, value);
            return this;
        }

        /**
         * API name: {@code runtime_mappings}
         * <p>
         * Adds an entry to <code>runtimeMappings</code> using a builder lambda.
         */
        public final Builder runtimeMappings(String key, Function<RuntimeField.Builder, ObjectBuilder<RuntimeField>> fn) {
            return runtimeMappings(key, fn.apply(new RuntimeField.Builder()).build());
        }

        /**
         * Builds a {@link FieldCapsRequest}.
         *
         * @throws NullPointerException
         *             if some of the required fields are null.
         */
        public FieldCapsRequest build() {
            _checkSingleUse();

            return new FieldCapsRequest(this);
        }
    }

    // ---------------------------------------------------------------------------------------------

    /**
     * Json deserializer for {@link FieldCapsRequest}
     */
    public static final JsonpDeserializer<FieldCapsRequest> _DESERIALIZER = ObjectBuilderDeserializer.lazy(
        Builder::new,
        FieldCapsRequest::setupFieldCapsRequestDeserializer
    );

    protected static void setupFieldCapsRequestDeserializer(ObjectDeserializer<FieldCapsRequest.Builder> op) {

        op.add(Builder::indexFilter, Query._DESERIALIZER, "index_filter");
        op.add(Builder::runtimeMappings, JsonpDeserializer.stringMapDeserializer(RuntimeField._DESERIALIZER), "runtime_mappings");

    }

    // ---------------------------------------------------------------------------------------------

    /**
     * Endpoint "{@code field_caps}".
     */
    public static final Endpoint<FieldCapsRequest, FieldCapsResponse, ErrorResponse> _ENDPOINT = new SimpleEndpoint<>(

        // Request method
        request -> {
            return "POST";

        },

        // Request path
        request -> {
            final int _index = 1 << 0;

            int propsSet = 0;

            if (ApiTypeHelper.isDefined(request.index())) propsSet |= _index;

            if (propsSet == 0) {
                StringBuilder buf = new StringBuilder();
                buf.append("/_field_caps");
                return buf.toString();
            }
            if (propsSet == (_index)) {
                StringBuilder buf = new StringBuilder();
                buf.append("/");
                SimpleEndpoint.pathEncode(request.index.stream().map(v -> v).collect(Collectors.joining(",")), buf);
                buf.append("/_field_caps");
                return buf.toString();
            }
            throw SimpleEndpoint.noPathTemplateFound("path");

        },

        // Request parameters
        request -> {
            Map<String, String> params = new HashMap<>();
            if (ApiTypeHelper.isDefined(request.expandWildcards)) {
                params.put("expand_wildcards", request.expandWildcards.stream().map(v -> v.jsonValue()).collect(Collectors.joining(",")));
            }
            if (request.ignoreUnavailable != null) {
                params.put("ignore_unavailable", String.valueOf(request.ignoreUnavailable));
            }
            if (request.allowNoIndices != null) {
                params.put("allow_no_indices", String.valueOf(request.allowNoIndices));
            }
            if (ApiTypeHelper.isDefined(request.fields)) {
                params.put("fields", request.fields.stream().map(v -> v).collect(Collectors.joining(",")));
            }
            if (request.includeUnmapped != null) {
                params.put("include_unmapped", String.valueOf(request.includeUnmapped));
            }
            return params;

        },
        SimpleEndpoint.emptyMap(),
        true,
        FieldCapsResponse._DESERIALIZER
    );
}
